/*
GmailSend Library for MetaTrader
Copyright (C) 2008 OpenThinking Systems, LLC
http://www.openthinkingsystems.com
All rights reserved.

- Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

- Neither the name of OpenThinking Systems LLC nor the names of its contributors
may be used to endorse or promote products derived from this software without
specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/
#include <time.h>
#include "EMail.h"
#include <iostream>


//constructors

// creates an empty EMail object
EMail::EMail()
: message(""), from(""), to_list(NULL), next_to(NULL)
{
}

// creates an EMail object initialized from the given file.
// If file doesn't exist or some other error, empty object
// is created.
EMail::EMail(string filename)
: message(""), from(""), to_list(NULL), next_to(NULL)
{
    //std::cout<< "Initialization from file not implemented. Empty EMail object created. \n";
}

//API

// Loads data from given file to the EMail object
int EMail::LoadFromFile(string filename)
{
    //std::cout<< "LoadFromFile(): Not implemented.\n";
    return 0;
}

// Fills the string 'msg' with the email content.
// Email 'content' is created on the fly by adding "headers" like
// from:, to:, subject: and date:, to the "message" text.
// Returns 0 on success, -1 on error.
int EMail::GetMessageA(std::string & msg)
{
    std::string tmp;
    std::string date;
    std::string rcpt;
    time_t t;

    // add the "Date:" header
    tmp = "Date: ";
    t = time(NULL);
    date = ctime(&t);
    tmp = tmp + date;

    // add the "From:" header
    tmp = tmp + "From: <" + this->from + ">\r\n";

    // add the "Subject:" header
    tmp = tmp + "Subject: " + this->subject + "\r\n";

    // add the "To:" header
    if(this->GetFirstRecepientTo(rcpt) < 0)
    {
        //cout<< "In EMail::GetMessage(): GetFirstRecepientTo() failed.\r\n";
        return -1;
    }
    tmp = tmp + "To: \" \" <" + rcpt + ">";
    while(rcpt != "")
    {
        if( 0 > this->GetNextRecepientTo(rcpt))
        {
            break;
        }
        tmp = tmp + ",\r\n \" \" <" + rcpt + ">";
    }
    tmp = tmp + "\r\n\r\n";

    // add the message content

    tmp = tmp + this->message;
    ////cout<< tmp;

    msg = tmp;
    return 0;
}

// Fills the string 'sender' with the email sender.
// Returns 0 on success, -1 on error.
int EMail::GetSender(string & sender)
{
    sender = this->from;
    return 0;
}

// Fills the string 'to' with the first 'TO' type of recepient
// Returns 0 on success; -1 on error
int EMail::GetFirstRecepientTo(string & to)
{
    if(this->to_list != NULL)
    {
        to = (this->to_list)->rcpt_address;
        return 0;
    }
    else
    {
        to = "";
        return -1;
    }
}

// Fills the string 'to' with the next 'TO' type of recepient
// Returns 0 on success; -1 when there are no more recepients left
// in the TO list. The next pointer is also reset to the 2nd TO
// recepient in such a case.
int EMail::GetNextRecepientTo(string & to)
{
    if(this->next_to != NULL)
    {
        to = (this->next_to)->rcpt_address;

        // increment the next_to pointer
        this->next_to = this->next_to->next_rcpt;

        return 0;
    }
    else
    {
        to = "";

        // reset the next_to pointer to the 2nd TO recepient
        this->next_to = this->to_list->next_rcpt;

        return -1;
    }
}

// Sets the email sender, returns 0 on success
int EMail::SetSender(string sender)
{
    this->from = sender;
    return 0;
}

// Sets the email subject, returns 0 on success
int EMail::SetSubject(string sub)
{
    this->subject = sub;
    return 0;
}

// Sets the email content, returns 0 on success
int EMail::SetMessage(string msg)
{
    this->message = msg;
    return 0;
}

// Adds a "TO" recepient, returns 0 on success
int EMail::AddRecepientTo(string to)
{
    struct recepient *temp;

    // adding first TO recepient
    if(this->to_list == NULL)
    {
        this->to_list = new struct recepient;
        this->to_list->rcpt_address = to;
        this->to_list->next_rcpt = NULL;
        return 0;
    }

    // adding second TO recepient
    if(this->to_list->next_rcpt == NULL)
    {
        this->next_to = new struct recepient;
        this->next_to->rcpt_address = to;
        this->next_to->next_rcpt = NULL;
        this->to_list->next_rcpt = this->next_to;
        return 0;
    }

    // adding subsequent TO recepients
    temp = next_to;
    while(temp->next_rcpt)
    {
        temp = temp->next_rcpt;
    }

    temp->next_rcpt = new struct recepient;
    temp = temp->next_rcpt;
    temp->rcpt_address = to;
    temp->next_rcpt = NULL;
    return 0;

}

