/*
GmailSend Library for MetaTrader
Copyright (C) 2008 OpenThinking Systems, LLC
http://www.openthinkingsystems.com
All rights reserved.

- Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

- Neither the name of OpenThinking Systems LLC nor the names of its contributors
may be used to endorse or promote products derived from this software without
specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/
#ifndef _EMail_h_
#define _EMail_h_

#include <string>

using namespace std;

class EMail
{
    private:
	string subject;		// email subject
	string message;		// email content
	string from;		// sender email address

	// common structure for a linked list of email recepients
	struct recepient
	{
	    string rcpt_address;
	    struct recepient *next_rcpt;
	};

	// list of "TO" recepients
	// This class may be later extended to have 'CC' and 'BCC' type
	// of recepients.
	struct recepient *to_list;

	// pointer to the next 'to' recepient in the 'to_list'. Used
	// by GetFirstToRecepient() and GetNextToRecepient() functions.
	struct recepient *next_to;

	// loads email from the given file. returns -1 on error, 0 otherwise
	int load_from_file(string filename);

    public:

    //constructors

	// creates an empty EMail object
    EMail();

	// creates an EMail object initialized from the given file.
	// If file doesn't exist or some other error, empty object
	// is created.
    EMail(string filename);

    //API

	// Loads data from given file to the EMail object
	int LoadFromFile(string filename);

	// Fills the string 'msg' with the email content.
        // Email 'content' is created on the fly by adding "headers" like
        // from:, to:, subject: and date:, to the "message" text.
	// Returns 0 on success, -1 on error.
	int GetMessageA(std::string & msg);

	// Fills the string 'sender' with the email sender.
	// Returns 0 on success, -1 on error.
	int GetSender(string & sender);

	// Fills the string 'to' with the first 'TO' type of recepient
	// Returns 0 on success; -1 on error; 1 on success when more than
	// one recepients are present.
	int GetFirstRecepientTo(string & to);

	// Fills the string 'to' with the next 'TO' type of recepient
	// Returns 0 on success; -1 on error; 1 on success when more
	// recepients are still left to be read.
	int GetNextRecepientTo(string & to);

	// Sets the email sender
	int SetSender(string sender);

	// Sets the email subject
	int SetSubject(string sub);

	// Sets the email content
	int SetMessage(string msg);

	// Adds a "TO" recepient
	int AddRecepientTo(string to);

};

#endif // _EMail_h_
