/*
GmailSend Library for MetaTrader
Copyright (C) 2008 OpenThinking Systems, LLC
http://www.openthinkingsystems.com
All rights reserved.

- Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

- Neither the name of OpenThinking Systems LLC nor the names of its contributors
may be used to endorse or promote products derived from this software without
specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>


#ifdef _WIN32
#include <winsock2.h>
#include <windows.h>
#define inet_pton(x,y,z)
#else
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <netdb.h>
#include <sys/socket.h>
#endif

#include <string.h>
#include <errno.h>


/**
    A function that opens an Internet socket connection.

    \param[in] server  A string like "www.google.com" - the URL of
                       the server to connect to.
    \param[in] port    Port number of the server to connect to
                       (for example, 25 for smtp port).

    \retval    socket_id    If everything OK.
    \retval    -1           If error. Errors are printed to stderr.
*/
int open_connection(char const *server, int const port)
{
    struct hostent *hp;
    struct sockaddr_in pin;
    int    sock;

    /* server hostname lookup */
    if ((hp = gethostbyname(server)) == NULL)
    {
        fprintf(stderr, "gethostbyname() returned NULL");
        return (-1);
    }

    /* fill in the socket structure with host information */
    memset(&pin, 0, sizeof(pin));
    pin.sin_family = AF_INET;
    pin.sin_addr.s_addr = ((struct in_addr *)(hp->h_addr))->s_addr;
    pin.sin_port = htons(port);

    /* grab an Internet domain socket */
    if ((sock = socket(AF_INET, SOCK_STREAM, 0)) == -1)
    {
        fprintf(stderr, "socket() returned -1");
        return (-1);
    }

    /* connect to 'port' on 'server' */
    if (connect(sock, (struct sockaddr *) &pin, sizeof(pin)) == -1)
    {
        fprintf(stderr, "connect() returned -1");
        perror("connect()");
        fprintf(stderr, "errno=%d\n", errno);
        fprintf(stderr, "%s\n", strerror(errno));
        return (-1);
    }

    /* return the socket */
    return sock;
}
