/*
GmailSend Library for MetaTrader
Copyright (C) 2008 OpenThinking Systems, LLC
http://www.openthinkingsystems.com
All rights reserved.

- Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

- Neither the name of OpenThinking Systems LLC nor the names of its contributors
may be used to endorse or promote products derived from this software without
specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

#include "ssl_cli.h"
#include <openssl/err.h>


BIO *bio_err = 0;
static char *pass;

static int  password_cb(char *buf, int num, int rwflag, void *userdata);
static void sigpipe_handle(int x);

//--------------------------------------
// STATIC FUNCTIONS
//--------------------------------------

/*The password code is not thread safe*/
static int password_cb(char *buf, int num, int rwflag, void *userdata)
{
    if(num<strlen(pass)+1)
        return(0);

    strcpy(buf,pass);
    return(strlen(pass));
}


static void sigpipe_handle(int x)
{
}

//--------------------------------------
// PUBLIC FUNCTIONS
//--------------------------------------

SSL_CTX *initialize_ctx(char *keyfile, char *password)
{
    SSL_METHOD *meth;
    SSL_CTX *ctx;

    if(!bio_err)
    {
        /* Global system initialization*/
        SSL_library_init();
        SSL_load_error_strings();

        /* An error write context */
        bio_err=BIO_new_fp(stderr,BIO_NOCLOSE);
    }

    /* Set up a SIGPIPE handler */
    //signal(SIGPIPE, sigpipe_handle);

    /* Create our context*/
    meth = SSLv23_method();
    ctx  = SSL_CTX_new(meth);

#if 0
    /* Load our keys and certificates*/
    if(!(SSL_CTX_use_certificate_chain_file(ctx, keyfile)))
        berr_exit("Can't read certificate file");

    pass = password;
    SSL_CTX_set_default_passwd_cb(ctx, password_cb);
    if(!(SSL_CTX_use_PrivateKey_file(ctx, keyfile, SSL_FILETYPE_PEM)))
        berr_exit("Can't read key file");

    /* Load the CAs we trust*/
    if(!(SSL_CTX_load_verify_locations(ctx, CA_LIST, 0)))
        berr_exit("Can't read CA list");
#endif // 0

#if (OPENSSL_VERSION_NUMBER < 0x00905100L)
    SSL_CTX_set_verify_depth(ctx, 1);
#endif

    return ctx;
}

void destroy_ctx(SSL_CTX *ctx)
{
    SSL_CTX_free(ctx);
}

/* A simple error and exit routine*/
int err_exit(char *string)
{
    fprintf(stderr, "%s\n", string);
    exit(0);
}

/* Print SSL errors and exit*/
int berr_exit(char *string)
{
    BIO_printf(bio_err, "%s\n", string);
    ERR_print_errors(bio_err);
    exit(0);
}

/* Check that the common name matches the
   host name*/
void check_cert(SSL *ssl, char *host)
{
    X509 *peer;
    char peer_CN[256];

    if(SSL_get_verify_result(ssl) != X509_V_OK)
        berr_exit("Certificate doesn't verify");

    /*Check the cert chain. The chain length
      is automatically checked by OpenSSL when
      we set the verify depth in the ctx */

    /*Check the common name*/
    peer = SSL_get_peer_certificate(ssl);
    X509_NAME_get_text_by_NID(X509_get_subject_name(peer),
                              NID_commonName,
                              peer_CN,
                              256);

    if(strcasecmp(peer_CN, host))
        err_exit("Common name doesn't match host name");
}

